@@ -377,12 +377,14 @@ int main(int argc, char* argv[])
         char* prompt = getenv("SAPHIRE_PROMPT");
         if(prompt) {
             string_obj* ret = STRING_NEW("");
-            if(saphire_shell3(ret, prompt, "SAPHIRE_PROMPT") == -1) {
+            sRFd* rfd = sRFd_new(0);
+            if(saphire_shell3(ret, prompt, "SAPHIRE_PROMPT", rfd) == -1) {
                 buf = readline("> ");
             }
             else {
                 buf = readline(string_c_str(ret));
             }
+            sRFd_delete(rfd);
 
             string_delete(ret);
         }
