@@ -132,6 +132,15 @@ hash_obj* gReadBuffers;
 vector_obj* gGlobalPipes;
     // スタッカブルグローバルパイプのデータ
 
+vector_obj* gQGlobalPipes;
+    // キューグローバルパイプの読み込みんだデータ
+
+string_obj* gGlobalPipe;
+    // グローバルパイプの読み込みんだデータ
+
+string_obj* gGlobalPipeNum[10];
+    // ナンバーグローバルパイプの読み込みんだデータ
+
 hash_obj* gRegexs;
     // 正規表現のキャッシュ
     // キーは正規表現の文字列
@@ -1753,15 +1762,8 @@ void sStatment_view(sStatment* self)
     else
         printf("self->mBackground false\n");
 
-    if(self->mGlobalPipeIn)
-        printf("self->mGlobalPipeIn true\n");
-    else
-        printf("self->mGlobalPipeIn false\n");
-
-    if(self->mGlobalPipeOut)
-        printf("self->mGlobalPipeOut true\n");
-    else
-        printf("self->mGlobalPipeOut false\n");
+    printf("self->mGlobalPipeIn %d\n", self->mGlobalPipeIn);
+    printf("self->mGlobalPipeOut %d\n", self->mGlobalPipeOut);
 }
 
 void sStatment_delete(sStatment* self)
@@ -1870,6 +1872,20 @@ void sStatment_save(sStatment* self, int fd)
         perror("write");
         exit(1);
     }
+
+    // mGlobalPipeInNum //
+    n = self->mGlobalPipeInNum;
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    // mGlobalPipeOutNum //
+    n = self->mGlobalPipeOutNum;
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
 }
 
 sStatment* sStatment_load(int fd)
@@ -1968,7 +1984,19 @@ sStatment* sStatment_load(int fd)
         exit(1);
     }
     self->mGlobalPipeOut = n;
-    
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    self->mGlobalPipeInNum = n;
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    self->mGlobalPipeOutNum = n;
+
     return self;
 }
 
@@ -2470,6 +2498,14 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
 void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script, BOOL run_user_runtimescript)
 {
     gGlobalPipes = VECTOR_NEW(10);
+    gQGlobalPipes = VECTOR_NEW(10);
+    gGlobalPipe = STRING_NEW("");
+    int i;
+    for(i=0; i<10; i++) {
+        gGlobalPipeNum[i] = STRING_NEW("");
+    }
+
+    // ナンバーグローバルパイプの読み込みんだデータ
 
     gAncestalMemoryPipeIn = STRING_NEW("");
     gAncestalMemoryPipeOut = STRING_NEW("");
@@ -2488,7 +2524,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gFileHandles = HASH_NEW(100);
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "0.9.9", 1);
+    setenv("SAPHIRE_VERSION", "1.0.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2578,6 +2614,17 @@ void saphire_final()
     }
     vector_delete(gGlobalPipes);
 
+    len = vector_size(gQGlobalPipes);
+    for(i=0; i<len; i++) {
+        string_delete(vector_item(gQGlobalPipes, i));
+    }
+    vector_delete(gQGlobalPipes);
+
+    string_delete(gGlobalPipe);
+    for(i=0; i<10; i++) {
+        string_delete(gGlobalPipeNum[i]);
+    }
+
     hash_it* it = hash_loop_begin(gFuncs);
     while(it != NULL) {
         sFunction_delete(hash_loop_item(it));
@@ -2823,6 +2870,44 @@ int sRFd_read_all(sRFd* self, string_obj* str)
     return 0;
 }
 
+// 0:正常終了 1:EOF -1:エラー
+int sRFd_read_all_preserve(sRFd* self, string_obj* str)
+{
+    if(self->mFd == -1) {
+        string_put(str, string_c_str(self->mBuffer));
+    }
+    else {
+        /// 読み込む
+        while(1) {
+            char buf[BUFSIZ+1];
+            int r = read(self->mFd, buf, BUFSIZ);
+
+            if(r < 0) {
+                return -1;
+            }
+            if(r == 0) {
+                if(string_length(str) == 0) {
+                    return 1;
+                }
+                break;
+            }
+            buf[r] = 0;
+
+            if(gKitutukiSigInt) {
+                gKitutukiSigInt = FALSE;
+                return -1;
+            }
+
+            string_push_back(self->mBuffer, buf);
+        }
+
+        /// 読み込んだバッファを書き込む
+        string_put(str, string_c_str(self->mBuffer));
+    }
+
+    return 0;
+}
+
 // 0:正常終了 1:EOF -1:エラー
 int sRFd_read_all_to_buffer(sRFd* self)
 {
@@ -2986,6 +3071,128 @@ int sRFd_read_oneline(sRFd* self, string_obj* str)
     }
 }
 
+// 0: 正常終了 1: EOF -1:エラー
+int sRFd_read_oneline_preserve(sRFd* self, string_obj* str)
+{
+    if(self->mFd == -1) {
+        string_obj* buffer = self->mBuffer;
+        if(gKitutukiSigInt) {
+            gKitutukiSigInt = FALSE;
+            return -1;
+        }
+
+        /// バッファーが無い
+        if(strcmp(string_c_str(buffer), "") == 0) {
+            return 1;
+        }
+
+        /// 取ったバッファから一行を得て返す ///
+        char *p;
+        int lf_char_size;
+        if(gLineField == kLF) {
+            p = strstr(string_c_str(buffer), "\n"); 
+            lf_char_size = 1;
+        }
+        else if(gLineField == kCRLF) {
+            p = strstr(string_c_str(buffer), "\r\n"); 
+            lf_char_size = 2;
+        }
+        else {
+            p = strstr(string_c_str(buffer), "\r"); 
+            lf_char_size = 1;
+        }
+
+        /// 改行がある
+        if(p) {
+            int n = p - string_c_str(buffer);
+            string_push_back3(str, string_c_str(buffer), n + lf_char_size);
+            return 0;
+        }
+        /// 改行が無い
+        else {
+            string_push_back(str, string_c_str(buffer));
+
+            return 0;
+        }
+    }
+    else {
+        while(1) {
+            /// 改行が含まれていればbreakする ///
+            if(gLineField == kCR) {
+                if(strstr(string_c_str(self->mBuffer), "\r")) {
+                    break;
+                }
+            }
+            else if(gLineField == kLF) {
+                if(strstr(string_c_str(self->mBuffer), "\n")) {
+                    break;
+                }
+            }
+            else {
+                if(strstr(string_c_str(self->mBuffer), "\r\n")) {
+                    break;
+                }
+            }
+
+            char buf[BUFSIZ+1];
+
+            int r = read(self->mFd, buf, BUFSIZ);
+
+            if(r < 0) {
+                return -1;
+            }
+            if(gKitutukiSigInt) {
+                gKitutukiSigInt = FALSE;
+                return -1;
+            }
+            buf[r] = 0;
+
+            /// EOF ///
+            if(r == 0) {
+                break;
+            }
+            else {
+                string_push_back(self->mBuffer, buf);
+            }
+        }
+
+        /// バッファーが無い
+        if(strcmp(string_c_str(self->mBuffer), "") == 0) {
+            return 1;
+        }
+
+        /// 取ったバッファから一行を得て返す ///
+        char *p;
+        int lf_char_size;
+        if(gLineField == kLF) {
+            p = strstr(string_c_str(self->mBuffer), "\n"); 
+            lf_char_size = 1;
+        }
+        else if(gLineField == kCRLF) {
+            p = strstr(string_c_str(self->mBuffer), "\r\n"); 
+            lf_char_size = 2;
+        }
+        else {
+            p = strstr(string_c_str(self->mBuffer), "\r"); 
+            lf_char_size = 1;
+        }
+
+        /// 改行がある
+        if(p) {
+            int n = p - string_c_str(self->mBuffer);
+            string_push_back3(str, string_c_str(self->mBuffer), n + lf_char_size);
+
+            return 0;
+        }
+        /// 改行が無い
+        else {
+            string_push_back(str, string_c_str(self->mBuffer));
+
+            return 0;
+        }
+    }
+}
+
 sWFd* sWFd_new(int fd)
 {
     sWFd* self = MALLOC(sizeof(sWFd));
